unit ParamsEditor;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Db, DBTables;

type
  TfrmParams = class(TForm)
    lbParams: TListBox;
    Label1: TLabel;
    Query1: TQuery;
    Label2: TLabel;
    cbDataType: TComboBox;
    Label3: TLabel;
    eValue: TEdit;
    chbNull: TCheckBox;
    btnOK: TButton;
    btnCancel: TButton;
    lDate: TLabel;
    lTime: TLabel;
    lDateTime: TLabel;
    lFloat: TLabel;
    procedure lbParamsClick(Sender: TObject);
    procedure eValueChange(Sender: TObject);
    procedure chbNullClick(Sender: TObject);
    procedure cbDataTypeChange(Sender: TObject);
  private
    { Private declarations }
    FParams:TParams;
    FNoUpdate:boolean;
  public
    { Public declarations }
  end;

  function EditParams(params:TParams):boolean;

const
  DataTypes:array [0..11] of record
     T:TFieldType;
     S:string;
  end = (
    (T:ftstring;          S:'String'),
    (T:ftInteger;         S:'Integer'),
    (T:ftsmallint;        S:'Smallint'),
    (T:ftWord;            S:'Word'),
    (T:ftBoolean;         S:'Boolean'),
    (T:ftCurrency;        S:'Currency'),
    (T:ftFloat;           S:'Float'),
    (T:ftBCD;             S:'BCD'),
    (T:ftDate;            S:'Date'),
    (T:ftTime;            S:'Time'),
    (T:ftDateTime;        S:'Datetime'),
    (T:ftBlob;            S:'Blob'));

implementation

{$R *.DFM}

function EditParams(params:TParams):boolean;
var
   i:integer;
   dt:TDateTime;
begin
     with TfrmParams.Create(nil) do
     begin
          lbParams.Clear;
          cbDataType.Clear;
          FParams:=params;

          dt:=now;
          lDate.caption:='Date:'+DateToStr(dt);
          lTime.caption:='Time:'+TimeToStr(dt);
          lDateTime.caption:='Datetime:'+DateTimeToStr(dt);
          lFloat.caption:='Float:'+FloatToStr(1234.56);

          for i:=0 to high(DataTypes) do
              cbDataType.Items.Add(DataTypes[i].S);

          for i:=0 to params.Count-1 do
              lbParams.Items.add(params.Items[i].Name);

          cbDataType.Enabled:=false;
          eValue.Enabled:=cbDataType.Enabled;
          chbNull.Enabled:=eValue.Enabled;

          Result:=ShowModal=mrOK;
          free;
     end;
end;

procedure TfrmParams.lbParamsClick(Sender: TObject);
var
   i:integer;
   n:integer;
   p:TParam;
begin
     n:=lbParams.ItemIndex;
     p:=FParams.ParamByName(lbParams.Items[n]);

     for i:=0 to high(DataTypes) do
     begin
//ShowMessage(format('i=%d, n=%d, DataTypes=(%s,%d), dt=%d',
//  [i,n,DataTypes[i].S,ord(DataTypes[i].T),ord(FParams[n].DataType)]));
         if DataTypes[i].T=p.DataType then
         begin
              FNoUpdate:=true;
              cbDataType.ItemIndex:=i;
              eValue.Text:=p.AsString;
              chbNull.Checked:=p.IsNull;

              cbDataType.Enabled:=true;
              eValue.Enabled:=cbDataType.Enabled;
              chbNull.Enabled:=eValue.Enabled;
              FNoUpdate:=false;
              exit;
         end;
     end;

     cbDataType.ItemIndex:=-1;
     eValue.Text:='';
     chbNull.State:=cbGrayed;
end;

procedure TfrmParams.eValueChange(Sender: TObject);
var
   n:integer;
begin
     if FNoUpdate then exit;
     n:=cbDataType.ItemIndex;
     if n<0 then exit;
     with FParams[lbParams.ItemIndex] do
     begin
          case DataTypes[n].T of
               ftstring:     AsString:=eValue.Text;
               ftInteger:    AsInteger:=StrToInt(eValue.Text);
               ftsmallint:   AsSmallInt:=StrToInt(eValue.Text);
               ftWord:       AsWord:=StrToInt(eValue.Text);
               ftBoolean:    AsBoolean:=trim(Uppercase(eValue.Text))='TRUE';
               ftCurrency:   AsCurrency:=StrToFloat(eValue.Text);
               ftFloat:      AsFloat:=StrToFloat(eValue.Text);
               ftBCD:        AsBCD:=StrToFloat(eValue.Text);
               ftDate:       AsDate:=StrToDate(eValue.Text);
               ftTime:       AsTime:=StrToTime(eValue.Text);
               ftDateTime:   AsDateTime:=StrToDateTime(eValue.Text);
               ftBlob:       ;
          end;
          chbNull.Checked:=IsNull;
     end;
end;

procedure TfrmParams.chbNullClick(Sender: TObject);
var
   n:integer;
begin
     if FNoUpdate then exit;
     n:=cbDataType.ItemIndex;
     if n<0 then exit;
     FNoUpdate:=true;
     if chbNull.Checked then
        with FParams[lbParams.ItemIndex] do
        begin
             Clear;
             Bound:=true;
             eValue.Text:='';
        end;
     FNoUpdate:=false;
end;

procedure TfrmParams.cbDataTypeChange(Sender: TObject);
var
   n:integer;
begin
     if FNoUpdate then exit;
     n:=cbDataType.ItemIndex;
     if n<0 then exit;
     with FParams[lbParams.ItemIndex] do
     begin
          FNoUpdate:=true;
          DataType:=DataTypes[n].T;
          eValue.Text:=AsString;
          chbNull.Checked:=IsNull;
          FNoUpdate:=false;
     end;
end;

end.
